pub mod migration;

use dioxus::prelude::*;

use crate::util::model::{member::Member, session::Session};

pub fn Members() -> Element {
    let members = use_resource(fetch_members);

    let modal_member: Signal<Option<Member>> = use_signal(|| None);

    rsx! {
        div {
            class: "max-w-3xl w-full",
            h2 { class: "font-bold text-lg", "Leden" },
            match &*members.read_unchecked() {
                Some(Ok(res)) => rsx! {
                    div {
                        class: "overflow-x-auto",
                        table {
                            class: "table table-zebra table-pin-rows",
                            thead {
                                tr {
                                    th { "Relatiecode" }
                                    th { "Naam" }
                                    th { "Registratiecode" }
                                }
                            }
                            tbody {
                                for member in res {
                                    MemberRow { member: member.clone(), modal_member: modal_member }
                                }
                            }
                        }
                    }
                },
                Some(Err(_)) => rsx! { div { "Error while loading members" } },
                None => rsx! { div { "Loading..." } },
            }
            MemberModal { member: modal_member }
        }
    }
}

#[component]
fn MemberRow(member: Member, modal_member: Signal<Option<Member>>) -> Element {
    let member_parse = member.clone();

    let registration_token = member_parse
        .registration_token
        .unwrap_or("None".to_string());

    rsx! {
        tr {
            class: "hover hover:cursor-pointer",
            onclick: move |_| {
                modal_member.set(Some(member.to_owned()))
            },
            th { "{member.id}" }
            td { "{member.name.full}" }
            td { "{registration_token}" }
        }
    }
}

#[server]
async fn fetch_members() -> Result<Vec<Member>, ServerFnError> {
    let user = Session::fetch_current_user().await?;

    if !user.admin {
        return Err(crate::Error::NoPermissions.into());
    }

    let members = Member::fetch_all().await?;
    Ok(members)
}

#[component]
fn MemberModal(member: Signal<Option<Member>>) -> Element {
    let mut member_groups = use_signal(|| vec![]);
    let mut loading = use_signal(|| false);

    use_effect(move || {
        if let Some(m) = member() {
            member_groups.set(m.groups);
        }
    });

    let submit = move |_| async move {
        loading.set(true);
        if let Some(m) = member() {
            if let Ok(_res) = update_groups(m.id, member_groups()).await {};
        };
        loading.set(false);
        member.set(None)
    };

    rsx! {
        dialog {
            class: "modal modal-bottom sm:modal-middle z-50",
            open: member().is_some(),
            div {
                class: "modal-box",
                h3 { class: "text-lg font-bold", "Lid bewerken" },
                if let Some(m) = member() {
                    MemberView { member: m, member_groups: member_groups }
                } else {
                    "Geen lid geselecteerd"
                }
                div {
                    class: "modal-action",
                    button {
                        class: "btn",
                        onclick: move |_| member.set(None),
                        disabled: loading(),
                        if loading() {
                            span { class: "loading loading-spinner" }
                        },
                        "Annuleren"
                    }
                    button {
                        class: "btn btn-primary",
                        onclick: submit,
                        disabled: loading(),
                        if loading() {
                            span { class: "loading loading-spinner" }
                        },
                        "Bewerken"
                    }
                }
            }
            label {
                class: "modal-backdrop bg-black/50",
                onclick: move |_| member.set(None)
            }
        }
    }
}

#[component]
fn MemberView(member: Member, member_groups: Signal<Vec<String>>) -> Element {
    let groups = use_context::<crate::Groups>();

    rsx! {
        div {
            div {
                class: "grid gap-3 sm:grid-flow-col sm:grid-rows-3 sm:grid-cols-2",
                label {
                    class: "form-control w-full",
                    div {
                        class: "label",
                        span { class: "label-text", "Naam" }
                    }
                    b { class: "select-all", "{member.name.full}" },
                }
                label {
                    class: "form-control w-full",
                    div {
                        class: "label",
                        span { class: "label-text", "Relatiecode" }
                    }
                    b { class: "select-all", "{member.id}" },
                }
                label {
                    class: "form-control w-full",
                    div {
                        class: "label",
                        span { class: "label-text", "Registratiecode" }
                    }
                    b { class: "select-all", {member.registration_token.unwrap_or("geen".to_string())} },
                }
                label {
                    class: "form-control w-full",
                    div {
                        class: "label",
                        span { class: "label-text", "Uur" }
                    }
                    b { class: "select-all", {member.hours.join(", ")} },
                }
                label {
                    class: "form-control w-full",
                    div {
                        class: "label",
                        span { class: "label-text", "Diploma" }
                    }
                    b { class: "select-all", {member.diploma.unwrap_or("geen".to_string())} },
                }
            }
            div {
                class: "mt-5",
                label {
                    class: "form-control w-full",
                    div {
                        class: "label",
                        span { class: "label-text", "Groepen" }
                    }
                    div {
                        class: "grid sm:grid-cols-2",
                        for (group_id, group_name) in groups.0 {
                            div {
                                class: "form-control",
                                label {
                                    class: "label cursor-pointer justify-start gap-2",
                                    input {
                                        r#type: "checkbox",
                                        class: "checkbox",
                                        checked: member_groups.read().contains(&group_id),
                                        oninput: move |event| {
                                            if event.value() == "true".to_string() {
                                                member_groups.push(group_id.clone());
                                            } else {
                                                member_groups.retain(|x| *x != group_id);
                                            }
                                        }
                                    }
                                    span { class: "label-text", "{group_name}" },
                                }
                            }
                        }
                    }
                }
            }
        }
    }
}

#[server]
async fn update_groups(member_id: String, groups: Vec<String>) -> Result<(), ServerFnError> {
    let user = Session::fetch_current_user().await?;

    if !user.admin {
        return Err(crate::Error::NoPermissions.into());
    }

    Member::set_groups(&member_id, groups).await?;

    Ok(())
}
