use dioxus::prelude::*;

use crate::util::model::{session::Session, user::User};

pub fn Users() -> Element {
    let users = use_resource(fetch_users);
    let modal_user: Signal<Option<User>> = use_signal(|| None);

    rsx! {
        div {
            class: "max-w-3xl w-full",
            h2 { class: "font-bold text-lg", "Gebruikers" },
            match &*users.read_unchecked() {
                Some(Ok(res)) => rsx! {
                    div {
                        class: "overflow-x-auto",
                        table {
                            class: "table table-zebra table-pin-rows",
                            thead {
                                tr {
                                    th { "Id" }
                                    th { "Email" }
                                }
                            }
                            tbody {
                                for user in res {
                                    UserRow { user: user.clone(), modal_user: modal_user }
                                }
                            }
                        }
                    }
                },
                Some(Err(_)) => rsx! { div { "Error while loading users" } },
                None => rsx! { div { "Loading..." } },
            }
            UserModal { user: modal_user }
        }
    }
}

#[component]
fn UserRow(user: User, modal_user: Signal<Option<User>>) -> Element {
    rsx! {
        tr {
            class: "hover hover:cursor-pointer",
            onclick: move |_| {
                modal_user.set(Some(user.to_owned()))
            },
            th { "{user.id}" }
            td { "{user.email}" }
       }
    }
}

#[component]
fn UserModal(user: Signal<Option<User>>) -> Element {
    let mut loading = use_signal(|| false);
    let mut user_admin = use_signal(|| false);

    use_effect(move || {
        if let Some(u) = user() {
            user_admin.set(u.admin);
        }
    });

    let submit = move |_| async move {
        loading.set(true);
        if let Some(u) = user() {
            if let Ok(_res) = update_user(u.id, user_admin()).await {};
        };
        loading.set(false);
        user.set(None)
    };

    rsx! {
        dialog {
            class: "modal modal-bottom sm:modal-middle z-50",
            open: user().is_some(),
            div {
                class: "modal-box",
                h3 { class: "text-lg font-bold", "Gebruiker bewerken" },
                if let Some(u) = user() {
                    UserView { user: u, admin: user_admin }
                } else {
                    "Geen gebruiker geselecteerd"
                }
                div {
                    class: "modal-action",
                    button {
                        class: "btn",
                        onclick: move |_| user.set(None),
                        disabled: loading(),
                        if loading() {
                            span { class: "loading loading-spinner" }
                        },
                        "Annuleren"
                    }
                    button {
                        class: "btn btn-primary",
                        onclick: submit,
                        disabled: loading(),
                        if loading() {
                            span { class: "loading loading-spinner" }
                        },
                        "Bewerken"
                    }
                }
            }
            label {
                class: "modal-backdrop bg-black/50",
                onclick: move |_| user.set(None)
            }
        }
    }
}

#[component]
fn UserView(user: User, admin: Signal<bool>) -> Element {
    rsx! {
        div {
            div {
                class: "grid gap-3",
                label {
                    class: "form-control w-full",
                    div {
                        class: "label",
                        span { class: "label-text", "Id" }
                    }
                    b { class: "select-all", "{user.id}" },
                }
                label {
                    class: "form-control w-full",
                    div {
                        class: "label",
                        span { class: "label-text", "Email" }
                    }
                    b { class: "select-all", "{user.email}" },
                }
            }
            div {
                class: "mt-5",
                label {
                    class: "form-control",
                    label {
                        class: "label cursor-pointer justify-start gap-2",
                        input {
                            r#type: "checkbox",
                            class: "toggle toggle-primary",
                            checked: admin(),
                            oninput: move |event| {
                                if event.value() == "true".to_string() {
                                    admin.set(true);
                                } else {
                                    admin.set(false);
                                }
                            }
                        },
                        span { class: "label-text", "Admin" },
                    }
                }
            }
        }
    }
}

#[server]
async fn fetch_users() -> Result<Vec<User>, ServerFnError> {
    let user = Session::fetch_current_user().await?;

    if !user.admin {
        return Err(crate::Error::NoPermissions.into());
    }

    let users = User::fetch_all().await?;

    Ok(users)
}

#[server]
async fn update_user(id: String, admin: bool) -> Result<(), ServerFnError> {
    let user = Session::fetch_current_user().await?;

    if !user.admin {
        return Err(crate::Error::NoPermissions.into());
    }

    User::update_with_id(&id, admin).await?;

    Ok(())
}
