#![allow(non_snake_case)]

mod components;
mod err;
mod util;

#[cfg(feature = "server")]
pub use err::Error;

use dioxus::prelude::*;

// Use routes
use components::admin::members::migration::Migration;
use components::admin::members::Members;
use components::admin::users::Users;
use components::admin::Admin;
use components::agenda::Agenda;
use components::home::Home;
use components::layout::AdminLayout;
use components::layout::Global;
use components::messages::create::MessagesCreatePage;
use components::messages::MessagesPage;
use components::settings::Settings;
use tracing::Level;

#[derive(Clone, Routable, Debug, PartialEq, serde::Serialize, serde::Deserialize)]
#[rustfmt::skip]
pub enum Route {
    #[layout(Global)]
        #[route("/")]
        Home {},
        #[route("/agenda")]
        Agenda {},
        #[route("/messages")]
        MessagesPage {},
        #[route("/messages/create")]
        MessagesCreatePage {},
        #[route("/settings")]
        Settings {},

        #[layout(AdminLayout)]
            #[route("/admin")]
            Admin {},
            #[route("/admin/users")]
            Users {},
            #[route("/admin/members")]
            Members {},
            #[route("/admin/members/migration")]
            Migration {},
}

#[cfg(feature = "server")]
#[tokio::main]
async fn main() {
    dioxus::logger::init(Level::INFO).expect("Failed to init logger");

    // Initialize surrealdb connection
    util::surrealdb::initialize()
        .await
        .expect("Error while initializing surrealdb");

    let address = dioxus_cli_config::fullstack_address_or_localhost();

    let router = axum::Router::new().serve_dioxus_application(ServeConfigBuilder::default(), App);

    let router = router.into_make_service();
    let listener = tokio::net::TcpListener::bind(address).await.unwrap();

    axum::serve(listener, router).await.unwrap();
}

#[cfg(not(feature = "server"))]
fn main() {
    dioxus::launch(App);
}

#[derive(Clone)]
pub struct Groups<'a>(Vec<(String, &'a str)>);

#[derive(Clone)]
pub struct Hours<'a>(Vec<(String, &'a str)>);

fn App() -> Element {
    let groups = Groups(vec![
        ("bestuur".to_string(), "Bestuur"),
        ("kader".to_string(), "Kader"),
        ("varendredden".to_string(), "Varend redden"),
        ("wedstrijden".to_string(), "Wedstrijden"),
        ("wedstrijden_trainer".to_string(), "Wedstrijden trainer"),
    ]);

    let hours = Hours(vec![
        ("a".to_string(), "A"),
        ("b".to_string(), "B"),
        ("c".to_string(), "C"),
        ("d".to_string(), "D"),
        ("e".to_string(), "E"),
        ("z".to_string(), "Z"),
        ("wedstrijd".to_string(), "Wedstrijd"),
    ]);

    use_context_provider(|| groups);
    use_context_provider(|| hours);

    rsx! {
        document::Stylesheet {
            href: asset!("assets/tailwind.css")
        }

        div {
            class: "h-screen flex flex-col",
            Router::<Route> {}
        }
    }
}
