#[cfg(feature = "server")]
use crate::util::surrealdb::{thing_to_string, DB};
use serde::{Deserialize, Serialize};
#[cfg(feature = "server")]
use surrealdb::sql::Thing;

#[cfg(feature = "server")]
mod migration;

#[derive(Debug, Deserialize, Serialize, PartialEq, Eq, Clone)]
pub struct Member {
    #[cfg_attr(feature = "server", serde(deserialize_with = "thing_to_string"))]
    pub id: String,
    pub name: Name,
    pub hours: Vec<String>,
    pub groups: Vec<String>,
    pub diploma: Option<String>,
    pub registration_token: Option<String>,
}

#[derive(Debug, Deserialize, Serialize, PartialEq, Eq, Clone)]
pub struct Name {
    pub first: String,
    pub full: String,
}

#[derive(Debug, Deserialize, Serialize, PartialEq, Clone)]
pub struct MembersMigration {
    pub inserted: Vec<Member>,
    pub updated: Vec<Member>,
    pub removed: Vec<Member>,
}

#[cfg(feature = "server")]
impl Member {
    pub async fn fetch_all() -> Result<Vec<Self>, surrealdb::Error> {
        let mut res = DB.query("SELECT id, name.first, name.full, hours, groups, diploma, registration_token FROM member").await?;

        res = res.check()?;

        let members: Vec<Self> = res.take(0)?;

        Ok(members)
    }

    pub async fn fetch_from_registration_token(
        registration_token: String,
    ) -> Result<Self, crate::Error> {
        let mut res = DB
            .query("SELECT id, name.first, name.full, hours, groups, diploma, registration_token FROM ONLY member WHERE registration_token = $registration_token LIMIT 1")
            .bind(("registration_token", registration_token))
            .await?;

        res = res.check()?;

        match res.take(0)? {
            Some(m) => Ok(m),
            None => Err(crate::Error::NoDocument),
        }
    }

    pub async fn fetch_from_user(user_id: &str) -> Result<Vec<Self>, crate::Error> {
        let mut res = DB
            .query("SELECT VALUE ->user_to_member->member.* FROM ONLY $user")
            .bind(("user", Thing::from(("user", user_id))))
            .await?;

        res = res.check()?;

        let members: Vec<Self> = res.take(0)?;

        Ok(members)
    }

    pub async fn set_groups(member_id: &str, groups: Vec<String>) -> Result<Self, crate::Error> {
        let mut res = DB
            .query("UPDATE $member SET groups = $groups")
            .bind(("groups", groups))
            .bind(("member", Thing::from(("member", member_id))))
            .await?;

        res = res.check()?;

        match res.take(0)? {
            Some(m) => Ok(m),
            None => Err(crate::Error::NoDocument),
        }
    }
}

impl MembersMigration {
    pub fn new() -> MembersMigration {
        MembersMigration {
            inserted: vec![],
            updated: vec![],
            removed: vec![],
        }
    }
}
