use serde::{Deserialize, Serialize};

#[cfg(feature = "server")]
use crate::util::surrealdb::{thing_to_string, DB};
#[cfg(feature = "server")]
use surrealdb::sql::statements::{BeginStatement, CommitStatement};

#[derive(Debug, Serialize, Deserialize)]
pub struct PushSubscription {
    endpoint: String,
    keys_256dh: String,
    keys_auth: String,
    encodings: String,
}

#[derive(PartialEq, Clone, Copy, Debug, Deserialize, Serialize, Eq)]
pub enum TargetKind {
    None,
    All,
    Group,
    Hourgroup,
    Hour,
    Member,
    Account,
    Day,
}

impl TargetKind {
    pub fn from_string(input: &str) -> Self {
        match input {
            "all" => Self::All,
            "group" => Self::Group,
            "hourgroup" => Self::Hourgroup,
            "hour" => Self::Hour,
            "member" => Self::Member,
            "account" => Self::Account,
            "day" => Self::Day,
            _ => Self::None,
        }
    }
}

#[derive(PartialEq, Debug, Deserialize, Serialize, Clone, Eq)]
pub struct Target {
    pub kind: TargetKind,
    pub value: String,
}

#[derive(Debug, Deserialize, Serialize, PartialEq, Eq, Clone)]
pub struct Message {
    #[cfg_attr(feature = "server", serde(deserialize_with = "thing_to_string"))]
    pub id: String,
    pub title: String,
    pub body: String,
    pub tagets: Vec<Target>,
}

#[cfg(feature = "server")]
impl Message {
    pub async fn create_proposal(
        title: String,
        body: String,
        targets: Vec<Target>,
    ) -> Result<usize, crate::Error> {
        let member_ids = fetch_members(targets).await?;

        Ok(member_ids.len())
    }

    pub async fn insert(
        title: String,
        body: String,
        channel: String,
        member_ids: Vec<String>,
    ) -> Result<(), crate::Error> {
        let mut transaction = DB
            .query(BeginStatement::default())
            .query("let $message = CREATE ONLY message SET title = $title, body = $body, channel = $channel RETURN id;")
            .bind(("title", title))
            .bind(("body", body))
            .bind(("channel", channel));

        for member_id in member_ids {
            transaction = transaction.query(format!("RELATE ONLY (type::thing('member', $member_id_{}))->member_to_message->(type::thing('message', $message.id));", member_id))
                .bind((format!("member_id_{}", member_id), member_id));
        }

        let res = transaction.query(CommitStatement::default()).await?;

        res.check()?;

        Ok(())
    }
}

/// Retrieve a vec of all the members that will recieve the message
#[cfg(feature = "server")]
async fn fetch_members(targets: Vec<Target>) -> Result<Vec<String>, surrealdb::Error> {
    let mut transaction = DB.query(BeginStatement::default());

    for target in targets {
        match target.kind {
            TargetKind::All => {
                transaction = transaction.query("SELECT VALUE meta::id(id) FROM member;");

                break;
            }
            _ => {}
        };
    }

    let mut res = transaction.query(CommitStatement::default()).await?;

    res = res.check()?;

    let members: Vec<String> = res.take(0)?;

    Ok(members)
}

#[cfg(feature = "server")]
async fn fetch_pushsubscriptions(
    targets: Vec<Target>,
) -> Result<Vec<PushSubscription>, surrealdb::Error> {
    let mut transaction = DB.query(BeginStatement::default());

    for target in targets {
        match target.kind {
            TargetKind::All => {
                transaction = transaction
                    .query("RETURN array::group(SELECT VALUE pushsubscriptions FROM user);");

                break;
            }
            _ => {}
        };
    }

    let mut res = transaction
        .query(CommitStatement::default())
        .await?
        .check()?;

    let pushsubscriptions: Vec<PushSubscription> = res.take(0)?;

    Ok(pushsubscriptions)
}
