use serde::{Deserialize, Serialize};

#[cfg(feature = "server")]
use crate::util::surrealdb::{thing_to_string, DB};
#[cfg(feature = "server")]
use surrealdb::sql::{
    statements::{BeginStatement, CommitStatement},
    Thing,
};

use super::message::PushSubscription;

#[derive(Debug, Deserialize, Serialize, PartialEq, Eq, Clone)]
pub struct User {
    #[cfg_attr(feature = "server", serde(deserialize_with = "thing_to_string"))]
    pub id: String,
    pub email: String,
    pub admin: bool,
    password: Option<String>,
}

#[cfg(feature = "server")]
impl User {
    pub async fn fetch_all() -> Result<Vec<Self>, surrealdb::Error> {
        let mut res = DB.query("SELECT id, email, admin FROM user").await?;

        res = res.check()?;

        let users: Vec<Self> = res.take(0)?;

        Ok(users)
    }

    pub async fn new(
        email: String,
        password: String,
        member_ids: Vec<String>,
    ) -> Result<Self, crate::Error> {
        // Create user record
        let mut transaction = DB.query(BeginStatement::default())
            .query("let $user = CREATE ONLY user SET email = $email, password = crypto::argon2::generate($password) RETURN id, email, admin;")
            .bind(("email", email))
            .bind(("password", password));

        // Link user with members
        for member_id in member_ids {
            transaction = transaction.query(format!("RELATE ONLY (type::thing('user', $user.id))->user_to_member->(type::thing('member', $member_id_{}));", member_id))
                .bind((format!("member_id_{}", member_id), member_id));
        }

        let mut res = transaction
            .query("RETURN $user")
            .query(CommitStatement::default())
            .await?;

        res = res.check()?;

        let user: Option<Self> = res.take(0)?;

        match user {
            Some(u) => Ok(u),
            None => Err(crate::Error::NoDocument),
        }
    }

    pub async fn relate_member(&self, member_id: &str) -> Result<(), crate::Error> {
        let res = DB
            .query("
                IF (SELECT id FROM ONLY user_to_member WHERE in = $user AND out = $member LIMIT 1) == NONE {
                    RELATE ONLY $user->user_to_member->$member;
                } ELSE {
                    THROW 'Relation already exists'
                };
            ")
            .bind(("user", Thing::from(("user", self.id.as_str()))))
            .bind(("member", Thing::from(("member", member_id))))
            .await?;

        res.check()?;

        Ok(())
    }

    pub async fn remove_relation(&self, member_id: &str) -> Result<(), crate::Error> {
        let res = DB
            .query(
                "
                IF (SELECT count() FROM user_to_member WHERE in = $user GROUP ALL)[0].count == 1 {
                    THROW 'At least one relation is required'
                } ELSE {
                    DELETE $user->user_to_member WHERE out = $member
                };
            ",
            )
            .bind(("user", Thing::from(("user", self.id.as_str()))))
            .bind(("member", Thing::from(("member", member_id))))
            .await?;

        res.check()?;

        Ok(())
    }

    pub async fn verify_credentials(
        email: String,
        password: String,
    ) -> Result<String, crate::Error> {
        let mut res = DB
            .query("SELECT id, email, admin FROM user WHERE email = $email AND crypto::argon2::compare(password, $password)")
            .bind(("email", email))
            .bind(("password", password))
            .await?;

        res = res.check()?;

        let user: Option<Self> = res.take(0)?;

        match user {
            Some(u) => Ok(u.id),
            None => Err(crate::Error::NoDocument),
        }
    }

    pub async fn change_email(&self, new_email: String) -> Result<String, crate::Error> {
        let mut res = DB
            .query("UPDATE $user SET email = $email RETURN VALUE email")
            .bind(("user", Thing::from(("user", self.id.as_str()))))
            .bind(("email", new_email))
            .await?;

        res = res.check()?;

        let email: Option<String> = res.take(0)?;

        match email {
            Some(e) => Ok(e),
            None => Err(crate::Error::NoDocument),
        }
    }

    pub async fn change_password(
        &self,
        old_password: String,
        new_password: String,
    ) -> Result<(), crate::Error> {
        let res = DB
            .query(
                "
                LET $user = (SELECT * FROM ONLY $user);

                IF crypto::argon2::compare($user.password, $old_password) {
                    UPDATE $user SET password = crypto::argon2::generate($new_password)
                } ELSE {
                    THROW 'Password dit not match'
                };
                ",
            )
            .bind(("user", Thing::from(("user", self.id.as_str()))))
            .bind(("old_password", old_password))
            .bind(("new_password", new_password))
            .await?;

        res.check()?;

        Ok(())
    }

    pub async fn update_with_id(id: &str, admin: bool) -> Result<(), crate::Error> {
        let res = DB
            .query("UPDATE $user SET admin = $admin;")
            .bind(("user", Thing::from(("user", id))))
            .bind(("admin", admin))
            .await?;

        res.check()?;

        Ok(())
    }

    pub async fn insert_pushsubscription(
        &self,
        pushsubscription: PushSubscription,
    ) -> Result<(), crate::Error> {
        let res = DB
            .query("UPDATE $user SET pushsubscriptions += $pushsubscription;")
            .bind(("user", Thing::from(("user", self.id.as_str()))))
            .bind(("pushsubscription", pushsubscription))
            .await?;

        res.check()?;

        Ok(())
    }
}
