use once_cell::sync::Lazy;
use serde::{Deserialize, Deserializer};
use surrealdb::engine::remote::ws::{Client, Ws};
use surrealdb::opt::auth::Root;
use surrealdb::sql::Thing;
use surrealdb::Surreal;

pub static DB: Lazy<Surreal<Client>> = Lazy::new(|| Surreal::init());

pub fn thing_to_string<'de, D>(deserializer: D) -> Result<String, D::Error>
where
    D: Deserializer<'de>,
{
    let t = Thing::deserialize(deserializer)?;
    Ok(t.id.to_raw())
}

pub async fn initialize() -> surrealdb::Result<()> {
    tracing::info!("Connectiong to surrealdb");

    DB.connect::<Ws>("localhost:8000")
        .await
        .expect("Failed to connect to surrealdb");

    DB.signin(Root {
        username: "root",
        password: "root",
    })
    .await?;

    DB.use_ns("wrbapp").use_db("wrbapp").await?;

    apply_queries().await.expect("Failed to apply queries");

    Ok(())
}

async fn apply_queries() -> surrealdb::Result<()> {
    // Define user table
    DB.query(
        "
            DEFINE TABLE OVERWRITE user SCHEMAFULL;

            DEFINE FIELD email ON TABLE user TYPE string
                VALUE string::lowercase($value)
                ASSERT string::is::email($value);
            DEFINE FIELD password ON TABLE user TYPE string;
            DEFINE FIELD admin ON TABLE user TYPE bool
                DEFAULT false;
            DEFINE FIELD pushsubscriptions ON TABLE user TYPE array<object>
                DEFAULT [];
            DEFINE FIELD pushsubscriptions.*.endpoint ON TABLE user TYPE string;
            DEFINE FIELD pushsubscriptions.*.keys_256dh ON TABLE user TYPE string;
            DEFINE FIELD pushsubscriptions.*.keys_auth ON TABLE user TYPE string;
            DEFINE FIELD pushsubscriptions.*.encodings ON TABLE user TYPE string;

            DEFINE INDEX userEmailIndex ON TABLE user COLUMNS email UNIQUE;
        ",
    )
    .await
    .expect("Failed to apply user query");

    DB.query(
        "
            DEFINE TABLE OVERWRITE user_to_member SCHEMAFULL TYPE RELATION FROM user TO member ENFORCED;
        ",
    )
    .await?;

    // Define member table
    DB.query(
        "
            DEFINE TABLE OVERWRITE member SCHEMAFULL;

            DEFINE FIELD id ON TABLE member TYPE record;
            DEFINE FIELD name ON TABLE member TYPE object;
            DEFINE FIELD name.first ON TABLE member TYPE string;
            DEFINE FIELD name.full ON TABLE member TYPE string;
            DEFINE FIELD registration_token ON TABLE member TYPE string
                VALUE rand::string(16);
            DEFINE FIELD diploma ON TABLE member TYPE option<string>;
            DEFINE FIELD hours ON TABLE member TYPE set<string>;
            DEFINE FIELD groups ON TABLE member TYPE set<string>;

            DEFINE INDEX memberIdIndex ON TABLE member COLUMNS id;
        ",
    )
    .await?;

    // Define session table
    DB.query(
        "
            DEFINE TABLE OVERWRITE session SCHEMAFULL;

            DEFINE FIELD user ON TABLE session TYPE record;
            DEFINE FIELD token ON TABLE session TYPE string
                VALUE rand::string(32);
            DEFINE FIELD expires ON TABLE session TYPE datetime
                VALUE time::now() + 1w;

            DEFINE INDEX sessionTokenIndex ON TABLE session COLUMNS token UNIQUE;
        ",
    )
    .await?;

    // Define messages table
    DB.query(
        "
            DEFINE TABLE OVERWRITE message SCHEMAFULL;

            DEFINE FIELD title ON TABLE message TYPE string;
            DEFINE FIELD body ON TABLE message TYPE string;
            DEFINE FIELD channel ON TABLE message TYPE string;

            DEFINE TABLE OVERWRITE member_to_message SCHEMAFULL TYPE RELATION FROM member TO message ENFORCED;
        ",
    )
    .await?;

    Ok(())
}
